#!/bin/bash
#@author Jacob Steelsmith 'jacob@steelsmith.org'
#@date 2008/04/29
#@file bridge-network.sh
#@version 0.1
#

#
#@brief
#Quick script to create a bridge, tunnel and TAP interface 
#for VirtualBox bridged networking support. 
#
#@detail
#This script will prompt for the required information to 
#setup a working tunnel, bridge and tap interface.
#
#It will detect the user running it and make them 
#the owner of the tunnel. It will then detect the current 
#IP, netmask and gateway, from ifconfig and route, in the case 
#of a static configuration.
#
#@todo
#detect and attempt to install required software.  
#detect DHCP configuration and configure accordingly
#incorporate groupship changes of /dev/net/tun
#incorporate group perm changes of /dev/net/tun
#incorporate the editing of /etc/udev/rules.d/20-names.rules ??? scary and 
#	poss ubuntu specific
#

#
#Copyright 2008 Jacob Steelsmith
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


DEBUG="0"	#debug output
RTO=10		#read timeout
DINT="eth0"	#default interface to bridge
DHCPD="N"	#default dhcp answer

#static interface values (the script detects the current settings...these are backup)
SIPD="192.168.0.100"	#ip
SNMD="255.255.255.0"	#netmask
SGWD="134.168.0.1"	#gateway

#make sure the script is being run as root
if [ "$(id -u)" != "0" ]; then
	echo "Sorry, you gotta sudo or be root."
	exit 1
fi

#let the user know what we're doing 

#GNU liscense stuff
echo
echo "bridge-network.sh  Copyright (C) 2008 Jacob Steelsmith. This program comes with ABSOLUTELY NO WARRANTY;"
echo
echo "This is free software distrributed unde the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version, and you are welcome to redistribute it under certain conditions."
echo
echo "For details see see <http://www.gnu.org/licenses/>."


echo
echo "This script will attempt create a TAP interface using a network bridge for VirtualBox."
echo

#debugging notification
if [ "$DEBUG" = 1 ]; then
	echo '[DEBUG] Debugging is turned on.';
fi

#get the user's primary interface to bridge.
echo "Please enter your primary interface. Defaulting to $DINT in $RTO seconds:"
read -t $RTO INTERFACE
echo

#check to see if we read something (if $INTERFACE is empty)
if [ -z "$INTERFACE" ]; then
	INTERFACE=$DINT
fi

#output the interface during debugging
if [ "$DEBUG" = 1 ]; then
	echo "[DEBUG] The interface is set to: $INTERFACE."
fi


#determine the default user name
if [ -n "$SUDO_USER" ]; then
	RUSER=$SUDO_USER
else
	RUSER=$USER
fi 


#get the username (default to current user)
echo "Please indicate the user that will own the new interface."
echo "Defaulting to $RUSER in $RTO seconds:"
read -t $RTO TOWNER
echo

#default the tap owner
if [ -z "$TOWNER" ]; then
	TOWNER=$RUSER
fi

#check the towner for an error...JIC
if [ -z "$TOWNER" ]; then
	echo "[ERROR] No TAP owner specified and could not determine default. Exiting."
	exit 1
fi

#output the tap owner during debugging
if [ "$DEBUG" = 1 ]; then
	echo "[DEBUG] The TAP owner is set to: $TOWNER."
fi

DHCP=0

#prompt until we get a Y or an N for dhcp configuration
while [[ "$DHCP" != "Y" && "$DHCP" != "N" ]]
do
	#check for dhcp
	echo "Is your current interface configured for DHCP? Y or N"
	echo "Defaulting to $DHCPD in $RTO seconds:"
	read -t $RTO DHCP
	echo
	
	if [[ -z "$DHCP" || "$DHCP" = 0 ]]; then 
		DHCP=$DHCPD
	else
		if [ "$DEBUG" = 1 ]; then
			echo "[DEBUG] Read $DHCP as the answer."
		fi
	fi 
	
	if [ "$DEBUG" = 1 ]; then
		echo "[DEBUG] DHCP configuration is set to $DHCP"
	fi

	if [[ "$DHCP" != "Y" && "$DHCP" != "N" ]]; then
		echo "[ERROR] DHCP configuration answer must be Y or N!"
		echo "Please try again."
	fi
done

#try to get the current ip address using ifconfig
CIP=ifconfig | grep 'inet addr:' | grep -v '127.0.0.1' | cut -d: -f2 | awk '{print $1}'

if [ -n "$CIP" ]; then
	SIPD=$CIP
fi

#try to get the current netmask using route
CNM=ifconfig | grep 'Mask:' | grep -v '127.0.0.1' | cut -d: -f4

if [ -n "$CNM" ]; then
	SNMD=$CNM
fi

#try to get the current default gateway using route
CDGW='route -n' | grep 'UG[ \t]' | awk '{print $2}'

if [ -n "$CDGW" ]; then
	SGWD=$CDGW
fi

#if dhcp is equal to no...we'll need to get the static information
if [ "$DHCP" = "N" ]; then
	
	echo "You have indicated manual configuration."
	echo "Please enter your ip address. Defaulting to $SIPD in $RTO seconds:"
	read -t $RTO SIP
	echo
	
	if [ -z "$SIP" ]; then
		SIP=$SIPD
	fi
	
	if [ "$DEBUG" = 1 ]; then
		echo "[DEBUG] Set static IP to $SIP."
	fi

	echo "Please enter your netmask. Defaulting to $SNMD in $RTO seconds:"
	read -t $RTO SNM
	echo
	
	if [ -z "$SNM" ]; then
		SNM=$SNMD
	fi

	if [ "$DEBUG" = 1 ]; then
		echo "[DEBUG] Set network mask to $SNM."
	fi

	echo "Please enter your default gateway. Defaulting to $SGWD in $RTO seconds:"
	read -t $RTO SGW
	echo

	if [ -z "$SGW" ]; then
		SGW=$SGWD
	fi

	if [ "$DEBUG" = 1 ]; then
		echo "[DEBUG] Set default gateway to $SGW."
	fi
	
else
	echo "You have opted for DHCP configuration."
fi

#create the TAP interface
echo "Using tunctl to create the interface tap1..."
if tunctl -t tap1 -u "$TOWNER" &> /tmp/bridge-network.log
then
	echo "Interface tap1 created!"
else
	#could do more here 
	echo "[ERROR] Error using tunctl to create interface tap1."
	echo "[ERROR] Please make sure tunctl is installed and in your path."
	echo "Dumping log to /var/log/bridge-network.log and exiting."
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

#add the bridge
echo "Using brctl to add the bridge br0..."
if brctl addbr br0 &> /tmp/bridge-network.log
then 
	echo "Bridge br0 created!"
else
	#do more here
	echo "[ERROR] Error using brctl to add the bridge br0."
	echo "[ERROR] Please make sure brctl is installed and in your path."
	echo "Dumping log to /var/log/bridge-network.log and exiting."
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

#turn promisc mode on nic
echo "Enabling promiscous mode on current interface using ifconfig..."
if ifconfig "$DINT" 0.0.0.0 promisc &> /tmp/bridge-network.log
then
	echo "Promiscous mode enabled!"
else
	echo "[ERROR] Promiscous mode failed. Exiting."
	echo "Dumping log to /var/log/bridge-network.log"
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

#add the primary interface to the bridge
echo "Adding your primary interface to your new bridge using brctl..."
if brctl addif br0 "$DINT" &> /tmp/bridge-network.log
then
	echo "Primary interface added to bridge!"
else
	echo "[ERROR] Failed to add $DINT to bro0."
	echo "Dumping log to /var/log/bridge-network.log and exiting."
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

echo "Configuring the bridge based on DHCP configuration using dhclient..."
if [ "$DHCP" = "Y" ]; then
	if dhclient br0 &> /tmp/bridge-network-dhcp.log
	then
		echo "DHCP configured for br0!"
		cat /tmp/bridge-network-dhcp.log
		cat /tmp/bridge-network-dhcp.log >>/tmp/bridge-network.log
		rm /tmp/bridge-network-dhcp.log
	else
		echo "[ERROR] Error configuring DHCP."
		echo "[ERROR] Please make sure dhclient is installed."
		cat /tmp/bridge-network-dhcp.log >>/tmp/bridge-network.log
		echo "Dumping log to /var/log/bridge-network.log and exiting."
		mv /tmp/bridge-network.log /var/log/bridge-network.log
		exit 1
	fi	
else
	echo "Setting static IP address and netmask for the bridge br0 using ifconfig..."
	if ifconfig br0 "$SIP" netmask "$SNM" &> /tmp/bridge-network.log
	then
		echo "Static IP address and netmask set for br0!"
	else
		echo "[ERROR] Error setting static IP address or netmask for bridge br0."
		echo "Dumping log to /var/log/bridge-network.log"
		mv /tmp/bridge-network.log /var/log/bridge-network.log
		exit 1
	fi

	echo "Setting the default gatway for the bridge br0 using route..."
	if route add default gw "$SGW" br0 &> /tmp/bridge-network.log
	then
		echo "Default gateway set for br0"
	else
		echo "[ERROR] Error setting default gatway for bridge br0."
		echo "Dumping log to /var/log/bridge-network.log"
		mv /tmp/bridge-network.log /var/log/bridge-network.log
		exit 1
	fi
fi

echo "Using brctl to add the TAP interface tap1 to the bridge br0..."
if brctl addif br0 tap1 &> /tmp/bridge-network.log
then
	echo "TAP interface tap1 added to bridge br0!"
else
	echo "[ERROR] Error adding TAP interface tap1 to bridge br0."
	echo "Dumping log to /var/log/bridge-network.log"
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

echo "Bringing the tap interface tap1 up using ifconfig..."
if ifconfig tap1 up &> /tmp/bridge-network.log
then
	echo "TAP interface tap1 brought up!"
else
	echo "[ERROR] Error bringing up tap1 using ifconfig."
	echo "Dumping log to /var/log/bridge-network.log"
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi

echo "Using chmod to change the rights to the tunnel."
if chmod 0666 /dev/net/tun &> /tmp/bridge-network.log
then
	echo "Operation completed!"
	echo "You should be able to use host networking now in VirtualBox!"
	echo "In the network settings change \"attached to\" to \"host interface\" and specify the interface name tap1."
	rm /tmp/bridge-network.log
	exit 0
else
	echo "[ERROR] Error changing permissions on /dev/net/tun to 0666."
	echo "Dumping log to /var/log/bridge-network.log"
	mv /tmp/bridge-network.log /var/log/bridge-network.log
	exit 1
fi
